#include "ring_buffer.h"

// Initialise le buffer circulaire
void ring_buffer_init(struct ring_buffer *rb) {
  rb->head = 0;
  rb->tail = 0;
}

// Ajoute un octet dans le buffer circulaire
void ring_buffer_put(struct ring_buffer *rb, uint8_t data) {
  rb->buffer[rb->head] = data;

  // Si le buffer est plein, on avance la queue
  // Ce qui est équivalent à se débarasser du plus ancien octet
  if (ring_buffer_is_full(rb)) {
    rb->tail = (rb->tail + 1) % RING_BUFFER_SIZE;
  }

  rb->head = (rb->head + 1) % RING_BUFFER_SIZE;
}

// Récupère un octet du buffer circulaire
uint8_t ring_buffer_get(struct ring_buffer *rb) {
  // Si il n'y a rien à lire, on retourne 0
  if (!ring_buffer_available_bytes(rb)) {
    return 0;
  }

  uint8_t data = rb->buffer[rb->tail];
  rb->tail = (rb->tail + 1) % RING_BUFFER_SIZE;

  return data;
}

// Indique si le buffer circulaire est vide
uint8_t ring_buffer_available_bytes(struct ring_buffer *rb) {
  if (rb->head >= rb->tail) {
    return rb->head - rb->tail;
  } else {
    return RING_BUFFER_SIZE - rb->tail + rb->head;
  }
}

// Indique si le buffer circulaire est plein
uint8_t ring_buffer_is_full(struct ring_buffer *rb) {
  return (rb->head + 1) % RING_BUFFER_SIZE == rb->tail;
}
